$fn = 72; // Auflösung Rundungen beim Rendern. 12 = schnell; 36 = gut genug für FDM 3DDruck

halter();

module halter(){
    // Parameter in mm
    GW = 2;     // Wandstärke (minimum)
    GR = GW-0.5;    // Radius abgerundete Ecken unten
    GRO = GW*4;     // Radius abgerundete Ecken oben
    
    k = 0.01; // Korrekturfaktor gegen Oberflächenartefakte
    kk = 2*k;
    
    // Maße des Geräts
    // "Rucksack" steht hinten über
    ERX = 26.5;   // Breite "Rucksack"
    ERY = 45.5;   // Höhe "Rucksack"
    ERZ = 4.75;   // Dicke "Rucksack"
    // 1/2TE = 8.75
    // 1 TE = 17.5
    // Gehäuse = 15
    // Gesamtdicke Halter = ERZ+EBZ
    
    ERXO = 6;   // X-Offset Rucksack, 8
    ERYO = 0;   // Y-Offset Rucksack
    ERZO = 0;
    
    // "Bauch" ist gleich dem Rucksack
    EBX = ERX;  // Breite
    EBY = ERY;  // Höhe
    EBZ = 4; // Dicke des Gehäuses
    
    EBXO = ERXO;
    EBYO = ERYO;
    EBZO = ERZ;
    
    // Klemmbereich mit schrägen Ecken
    EKX = 8; // Ausschnittbreite
    EKY = 31;   // Ausschnitthöhe
    EKZ = EBZ;  // gleiche Dicke
    EKXO = ERXO-EKX;   // X-Offset Außenkante
    EKYO = (EBY-EKY)/2; // Y Offset
    EKZO = ERZ;
    EKD = sqrt(((EBY-EKY)/2)^2+EKX^2);
    EKDH = sqrt(EKD^2*2);
    
    // Oberseite
    EOX = 11;
    EOY = 22;
    EOZ = EBZ;
    EOXO = EKX+EBX+EKXO;
    EOYO = (EBY-EOY)/2;
    EOZO = ERZ;
    EOD = sqrt(((EBY-EOY)/2)^2+EOX^2);
    EODH = sqrt(EOD^2*2);
    
    // Aussparungen Montagelaschen
    EMX = 12;
    EMY = 5;
    EMZ = EBZ; 
    EMXO = EBXO+7;      // X-Offset
    EMYOU = EBYO-EMY;   // Y-Offset "untere" Lasche
    EMYOO = EBYO+EBY;   // Y-Offset "obere" Lasche
    EMZO = EBZO;        // Z-Offset
    
    // Außenmaße errechnen
    GX = 2*GW+EOX+EBX+EKX+EKXO; // X-Höhe des Gehäuses
    GY = 2*GW+EBY+2*EBYO;   // Aufbauhöhe über Hutschiene, ohne Haltelaschen
    GZ = EBZ+ERZ;           //  des Gehäuse
    
    // Haltenasen DIN 35mm Schiene
    //Feste Seite
    HFX = 8;
    HFY = 5;
    HFZ = GZ;
    HFXD = sqrt(HFY^2*2);
    //Lasche
    HLXO = HFX-HFXD/2+36;
    HLXN = HLXO-2;
    HLYN = 1;
    HLX = 2*GW+GX-HLXO;
    HLY = HFY/2;
    HLZ = GZ;
    HLFX = GW;
    HLFY = EOYO+EBYO+GW;
    HLFZ = GZ;
    //Hebel zum entriegeln
    HLHX = GW;
    HLHY = HLY;
    HLHZ = GZ;
    HLHXO = HLXN+HLX+2*GW-HLHX;
    HLHYO = 0;
    
    echo(GX, GY, GZ);
    
    // Grundform
    difference(){
        union(){
            translate([0,0,0]) cube([GX, GY-GRO, GZ]);
            translate([GRO,0,0]) cube([GX-GRO*2, GY, GZ]);
            // Abrundung oben
            translate([GRO,GY-GRO,0]) cylinder(h=GZ, r=GRO);
            translate([GX-GRO,GY-GRO,0]) cylinder(h=GZ, r=GRO);
        }
        // Aussparung für "Rucksack"
        translate([GW+ERXO,GW+ERYO,ERZO-k]) cube([ERX, ERY, GW+ERZ+k]);
        // Aussparung für "Bauch"
        translate([GW+EBXO,GW+EBYO,EBZO]) cube([EBX,EBY,EBZ+k]);
        // Aussparung Klemmbereich
        translate([EKXO,GW+EBYO+EKYO,EBZO]) cube([EKX+GW+k,EKY,EKZ+k]);
        translate([GW+EBXO,GW+EBYO,EBZO]) rotate([0,0,45]) cube([EKD,EKD,EKZ+k]);
        translate([GW+EBXO,GW+EBYO+2*EKYO+EKY-EKDH,EBZO]) rotate([0,0,45]) cube([EKD,EKD,EKZ+k]);
        // Aussparung Oberseite
        translate([GW+EOXO,GW+EBYO+EOYO,EOZO]) cube([EOX,EOY,EOZ+k]);
        translate([GW+EOXO,GW+EBYO,EOZO]) rotate([0,0,45]) cube([EOD,EOD,EOZ+k]);
        translate([GW+EOXO,GW+EBYO+2*EOYO+EOY-EODH,EOZO]) rotate([0,0,45]) cube([EOD,EOD,EOZ+k]);
        // Aussparung Montagelaschen
        translate([GW+EMXO,GW+EMYOU+k,EMZO]) cube([EMX,EMY+2*k,EMZ+k]);
        translate([GW+EMXO,GW+EMYOO-k,EMZO]) cube([EMX,EMY+2*k,EMZ+k]);
        // Aussparung Rastlasche
        translate([EOXO+GW*3,0,-k]) rotate([0,0,-45]) cube([EOD,EOYO+GW,GZ+kk]);
    }
    
    // Hutschienensockel
    // feste Lasche
    difference(){
        union(){
            translate([0,GR-HFY,0]) cube([HFX,HFY+GW-GR,HFZ]);
            translate([GR,-HFY,0]) cube([HFX-GR,HFY+GW,HFZ]);
            translate([GR,-HFY+GR,0]) cylinder(h=GZ, r=GR);
        }
        translate([HFX,-HFXD/1.8,-k]) rotate([0,0,45]) cube([HFY,HFY,HFZ+kk]);
    }
    
    // Federlasche
    difference(){
        union(){
            // Rastnase
            translate([HLXN,GR-HLY,0]) cube([HLX+2*GW,HLY-GR,HLZ]);
            translate([HLXN,-HLY,0]) cube([HLX+2*GW-GR,HLY,HLZ]);
            // Abrundung Außenecke
            translate([HLXN+HLX+2*GW-GR,-HLY+GR,0]) cylinder(h=GZ, r=GR);
            // Federsteg
            translate([GX,0,0]) cube([HLFX, HLFY, HLFZ]);
            // Abrundung Federsteg
            translate([GX+GW-GR,HLFY,0]) cylinder(h=GZ, r=GR);
            // Hebellasche
            translate([HLHXO,HLHYO,0]) cube([HLHX, HLHY, HLHZ]);
        }
        translate([HLXN,-HLY*1.9,-k]) rotate([0,0,40]) cube([HLY,HLY*1.5,HLZ+kk]);
        translate([HLXN,-HFXD/3+HLYN*0.9,-k]) rotate([0,0,30]) cube([HLY*1.5,HLY,HLZ+kk]);
    }
}